<?php
/**
* Project.Extract
* Main Class
* @version 2.2
* @copyright Nighthawk
*/

require 'text.php';
require 'connect.php';
require 'addon.php';
require 'notifications.php';

define('CONFIG_FILE', 'config.pe');
$cfg->read(CONFIG_FILE);
define('INSTALLED', $cfg->config['install']);
define('FILEFLAG', $cfg->config['fileflag']);
define('OUTPUT', $cfg->config['output']);

class Main extends ConfigParse
{
	var $title;
	var $project_files = array("libs/text.php", "libs/main.php", "libs/config.php", "libs/connect.php", "libs/config-parser.php", "libs/addon.php", "libs/main.pe", "libs/notifications.php");
	var $errorParse = false;
	var $errorMessage = array();
	var $logDirExist = false;
	var $coreFileExist = false;

	//code
	var $overwrite = 0;
	var $results = 0;
	var $deletelogs = false;

	public function __construct($pTitle){

		$this->title = $pTitle;
		self::checkDirectories();
		self::checkLogDir();
		//install
		if(isset($_GET['install'])){
			if($_GET['install'] == 'logs'){
				self::install('logs');
				log::debug('Re-installing Log Directory');
			}elseif($_GET['install'] == 'all'){
				self::install('all');
				log::debug('Re-installing');
			}else{
				header("location: ".INDEX);
			}
			if(CONNECTED){
				log::debug('Collecting Anonymous Data');
				self::openLink(project_installed_link, WEBINDEX, $_SERVER['SCRIPT_FILENAME']);
			}else{
				header('location: '.INDEX);
			}
		}
	}

	protected function checkDirectories(){
		foreach ($this->project_files as $key) {
			if(!file_exists($key)){
				self::addError("File '<b>".$key."</b>' not found, please re-install.");
				$this->errorParse = true;
				$this->coreFileExist = false;
			}else{
				$this->coreFileExist = true;
			}
		}
	}
	protected function checkLogDir(){
		if(file_exists('logs/')){
			$this->logDirExist = true;
		}else{
			$this->logDirExist = false;
			self::addError("Logs directory not found! <a href=\"./?install=logs\" title=\"Re-install\">Click here</a> to install it.");
		}
	}
	protected function install($specify=0){
		if($specify == 'logs'){
			if(!file_exists('logs/')){
				mkdir('logs/');
			}
		}elseif($specify == 'all'){
			if(!file_exists('logs/')){
				mkdir('logs/');
			}
			$data = array(
				'fileflag' => 'a',
				'output' => 'detailed',
				'install' => 'true',
				);
			$this->write($data, CONFIG_FILE);
		}
	}
	//OpenLink Function with a GET returnLink
	protected function openLink($link, $returnLink, $installed_link){
		header( 'location: ' . $link . '&rl=' . $returnLink . '&il=' . $installed_link);
	}

	public function error(){
		if($this->errorParse == true){
			?>
			<div class="col-1 span-4 message-container">
				<div class="span-2">
					<h3><i class="icon-exclamation"></i> Error!</h3>
					<?php
						foreach ($this->errorMessage as $key) {
							echo "<p class=\"message\"> $key <i class=\"icon-cross\"></i></p>";
						}
					?>
				</div>
			</div>
			<?php
		}
	}
	public function addError($errorMsg=0){
		log::error($errorMsg);
		$this->errorParse = true;
		array_push($this->errorMessage, $errorMsg);
	}

	public function isInstalled(){
		if(INSTALLED == 'true'){
			log::debug('Installed')
			?><h6 style="color:rgba(255,255,255,0.9);font-family:'OpenSans Light Italic';"><i class="icon-check" style="color:rgba(100,255,180,0.9);"></i> Installed</h6><?php
		}elseif(INSTALLED == 'false'){
			log::error('Not Installed')
			?><h6 style="color:rgba(255,255,255,0.9);font-family:'OpenSans Light Italic';"><i class="icon-cross" style="color:rgba(255,80,80,0.9);"></i> Not Installed</h6><?php
		}
	}
	public function logDirExists(){
		if(INSTALLED == 'true' && $this->logDirExist){
			log::debug('Log Directory Exists')
			?><h6 style="color:rgba(255,255,255,0.9);font-family:'OpenSans Light Italic';"><i class="icon-check" style="color:rgba(100,255,180,0.9);"></i> Log Directory Found</h6><?php
		}elseif(INSTALLED == 'true' && $this->logDirExist == false){
			log::error('Log Directory Doesn\'t Exist')
			?><h6 style="color:rgba(255,255,255,0.9);font-family:'OpenSans Light Italic';"><i class="icon-cross" style="color:rgba(255,80,80,0.9);"></i> Log Directory Not Found</h6><?php
		}
	}
	public function allFilesExist(){
		if($this->coreFileExist == true){
			log::debug('Core File(s) exist');
			?><h6 style="color:rgba(255,255,255,0.9);font-family:'OpenSans Light Italic';"><i class="icon-check" style="color:rgba(100,255,180,0.9);"></i> Core File(s) Loaded</h6><?php
		}elseif($this->coreFileExist == false){
			log::error('Core File(s) Missing');
			?><h6 style="color:rgba(255,255,255,0.9);font-family:'OpenSans Light Italic';"><i class="icon-cross" style="color:rgba(255,80,80,0.9);"></i> Core File(s) Not Loaded</h6><?php
		}
	}


	public function setConfig($overwrite, $results, $deletelogs){
		$this->overwrite = $overwrite;
		$this->results = filter_var($results, FILTER_SANITIZE_STRING);
		$this->deletelogs = $deletelogs;
		$data = array(
			'fileflag' => $this->overwrite,
			'output' => $this->results,
			'install' => INSTALLED,
			);
		$this->write($data, CONFIG_FILE);
		log::debug('Writing to config: fileflag='.$this->overwrite.', output='.$this->results.', install='.INSTALLED);
		if($this->deletelogs == true){
			if(is_dir('logs')){
				Addon::deleteDir('logs/');
				log::debug('Deleting CS2D Log Files');
			}
		}
		header('location: '.INDEX);
	}

}
?>