<?php
/**
* Project.Extract
* Text Library
* @version 2.2
* @copyright Soushiant & Nighthawk 2014
*/


define('FILTER_USGN', 'usgn');
define('FILTER_CONSOLE', 'svlogs');
define('FILTER_TEXT', 'findtext');
define('TIMESTAMP_LEN', 11);
define('USER_MATCH_TERM', 'usgn:');
define('NORMAL_TEXT', 'normal_text');
define('LOWER_TEXT', 'lower_text');
define('SEARCH_ALL', 'search_all');
define('SEARCH_ACCURATE', 'search_accurate');


class Text
{
	var $type;
	var $string;
	var $linecount = 0;
	var $lc = 0;
	var $file = '';
	var $logType = NULL;
	protected $filterArr = array('Use quotation marks for parameters with blanks. e.g.: mp_unbuyable "Tactical Shield', 'switching to ingame mode...');
	protected $consoleArr = array('UDP socket initialized using port ', 'U.S.G.N.: ', 'Connecting to ', 'load map \'', 'receiving server data ', 'receiving player data ', 'receiving hostage data ', 'receiving item data ', 'receiving entity states ', 'receiving dynamic object data ', 'receiving projectile data ', 'receiving dynamic object image data ', 'receiving tween data ', 'receiving custom tile data ', 'receiving final ACK ', 'switching to ingame mode...', '----- Server joined -----', 'LUA ERROR: ', 'ERROR: ');

	public function __construct($type, $string=NULL, $term=NULL, $cs=NULL)
	{
		self::checkFiles();
		if(!empty($type)){
		 	$this->type = $type;
		 	switch ($this->type) {
		 		case FILTER_USGN:
		 			self::filter_usgn();
		 			break;
		 		case FILTER_CONSOLE:
		 			self::filter_console();
		 			break;
		 		case FILTER_TEXT:
		 			$this->string = $string;
		 			self::filter_text($this->string, $term, $cs);
		 			break;
		 		
		 		default:
		 			log::error('Text Module: Filter Method Not Specified');
		 			break;
		 	}
		}else{
			log::error('Error: Type is NULL');
		}
	}

	protected function checkFiles()
	{
		if(!file_exists($GLOBALS['paths']['search'])){
			$cf = fopen($GLOBALS['paths']['search'], 'a+');
			log::debug('Creating File: '.$GLOBALS['paths']['search']);
			fclose($cf);
		}
		if(!file_exists($GLOBALS['paths']['users'])){
			$cf = fopen($GLOBALS['paths']['users'], 'a+');
			log::debug('Creating File: '.$GLOBALS['paths']['users']);
			fclose($cf);
		}
		if(!file_exists($GLOBALS['paths']['svlog'])){
			$cf = fopen($GLOBALS['paths']['svlog'], 'a+');
			log::debug('Creating File: '.$GLOBALS['paths']['svlog']);
			fclose($cf);
		}
	}
	protected function filter_console(){
		$f = scandir($GLOBALS['paths']['logs']);
		log::debug('Extracting Console');
		for($i=2;$i<count($f);$i++)
		{
			$file = fopen($GLOBALS['paths']['logs'].'/'.$f[$i], 'r');
			if($file)
			{
				$this->file = $f[$i];
				log::debug('Reading '.$this->file);
				while(($line = fgets($file)) !== false)
				{
					$this->linecount = $this->linecount + substr_count($line, PHP_EOL);
					for ($i=0; $i < count($this->consoleArr); $i++) { 
						if(preg_match('/^'.$this->consoleArr[$i].'/i', substr($line, TIMESTAMP_LEN)));
						{
							$output = fopen($GLOBALS['paths']['svlog'], FILEFLAG);
							$this->lc++;
							//filter crap
							fputs($output, self::filterCrap($line)."\n");
							self::extractData(self::filterCrap($line));
						}	
					}
				}
				$this->linecount=0;
				fclose($file);
			}else{
				log::error('Error Opening: '.$f[$i]);
			}
		}
	}
	protected function filter_usgn()
	{
		$f = scandir($GLOBALS['paths']['logs']);
		log::debug('Extracting User Information');
		for($i=2;$i<count($f);$i++)
		{
			$file = fopen($GLOBALS['paths']['logs'].'/'.$f[$i], 'r');
			if($file)
			{
				$this->file = $f[$i];
				log::debug('Reading '.$this->file);
				while(($line = fgets($file)) !== false)
				{
					$this->linecount = $this->linecount + substr_count($line, PHP_EOL);
					if(preg_match('/'.USER_MATCH_TERM.'/', strtolower($line)))
					{
						$output = fopen($GLOBALS['paths']['users'], FILEFLAG);
						$this->lc++;
						//filter crap
						fputs($output, self::filterCrap($line)."\n");
						self::extractData(self::filterCrap($line, LOWER_TEXT));
					}
				}
				$this->linecount=0;
				fclose($file);
			}else{
				log::error('Error Opening: '.$f[$i]);
			}
		}
	}


	protected function filterCrap($string, $case=NORMAL_TEXT){
		//filter timestamp
		$string = substr($string, TIMESTAMP_LEN);
		//filter case
		switch ($case) {
			case NORMAL_TEXT:
				//nothin
				break;
			case LOWER_TEXT:
				strtolower($string);
				break;
		}
		//filter numbering
		for($i=1;$i<=32;$i++){
			if(preg_match('/#'.$i.':/', $string)){
				if($i<10){
					$string = substr($string, 3);
				}else{
					$string = substr($string, 4);
				}
			}
		}
		//return dat shiet
		return trim($string);
	}

	private $dataList = array();
	protected function extractData($string){
		//echo "<a href=\"#\">".$usgn."</a><br/>";
		//check log type
		if(preg_match('/level:/', $string) && !preg_match('/Kicked/i', $string) && !preg_match('/Banned/i', $string) && preg_match('/usgn:/', $string) && preg_match('/ip:/', $string)){
			/**
			 *	This is the HC Log type
			 *	Format: <name>, level: <level>, usgn: <usgn>, ip: <ip>
			 */
			$this->logType = 'HC Log Type';
			//Break the String up
			//	Start
			$stringStart = substr($string, 0, strpos($string, 'usgn:'));
			//	USGN
			$USGNstart = strpos($string, 'usgn:') + strlen('usgn: ');
			$USGNend = strpos($string, ', ip:') - strlen($string);
			$usgn = substr($string, $USGNstart, $USGNend);
			//	Check for Valid USGN
			if(!$usgn == 0){
				$usgn = '<a target=\'_blank\' href='.usgn_link.$usgn.' class=\'link\'>'.$usgn.'</a>';
			}else{
				$usgn = '<span class="no-usgn">none</span>';
			}
			//	String End //IP
			$IPstart = strpos($string, 'ip:') + strlen('ip:');
			$ip = substr($string, $IPstart);
			$this->dataList['start'] = $stringStart;
			$this->dataList['usgn'] = $usgn;
			$this->dataList['ip'] = $ip;
			// Echo dat shit.
			echo '<div class="line-number col-1-2">'.$this->lc.'</div><div class="line col-2-2">'.$this->dataList['start'].' usgn: '.$this->dataList['usgn'].', ip: '.$this->dataList['ip'].'<span class=\'finfo\'> ( '.$this->file.' :: '.$this->linecount.' :: <span class\'l-type\'>'.$this->logType.'</span> )</div><br/>';

		}elseif(!preg_match('/level:/', $string) && !preg_match('/Kicked/i', $string) && !preg_match('/Banned/i', $string) && preg_match('/USGN:/', $string) && preg_match('/IP:/', $string)){
			/**
			 *	This is the LSD Log type
			 *	Format: Name: <name> IP: <ip> USGN: <usgn>
			 */
			$this->logType = 'LSD Log Type';
			$stringStart = substr($string, 0, strpos($string, 'IP:'));
			$USGNstart = strpos($string, 'USGN:') + strlen('USGN: ');
			$USGNend = strlen($string);
			$usgn= substr($string, $USGNstart);
			if(!$usgn == 0){
				$usgn = '<a target=\'_blank\' href='.usgn_link.$usgn.' class=\'link\'>'.$usgn.'</a>';
			}else{
				$usgn = '<span class=\'no-usgn\'>none</span>';
			}
			$IPstart = strpos($string, 'IP:') + strlen('IP:');
			$IPend = strpos($string, 'USGN:') - strlen($string);
			$ip = substr($string, $IPstart, $IPend);
			$this->dataList['start'] = $stringStart;
			$this->dataList['usgn'] = $usgn;
			$this->dataList['ip'] = $ip;
			// Echo dat shit.
			echo '<div class="line-number col-1-2">'.$this->lc.'</div><div class="line col-2-2">'.$this->dataList['start'].', usgn: '.$this->dataList['usgn'].', ip: '.$this->dataList['ip'].'<span class=\'finfo\'> ( '.$this->file.' :: '.$this->linecount.' :: <span class\'l-type\'>'.$this->logType.'</span> )</div><br/>';
		}elseif(!preg_match('/level:/', $string) && !preg_match('/Kicked/i', $string) && !preg_match('/Banned/i', $string) && preg_match('/usgn:/', $string) && preg_match('/ip:/', $string)){
			/**
			 *	This is the -[IFwsI]- HC Log Type
			 *	Format: name: <name>, usgn: <usgn>, ip: <ip>
			 */
			$this->logType = '-[IFwsI]- HC Log Type';
			//Break the String up
			//	Start
			$stringStart = substr($string, 0, strpos($string, 'usgn:'));
			//	USGN
			$USGNstart = strpos($string, 'usgn:') + strlen('usgn: ');
			$USGNend = strpos($string, ' , ip:') - strlen($string);
			$usgn = substr($string, $USGNstart, $USGNend);
			//	Check for Valid USGN
			if(!$usgn == 0){
				$usgn = '<a target=\'_blank\' href='.usgn_link.$usgn.' class=\'link\'>'.$usgn.'</a>';
			}else{
				$usgn = '<span class="no-usgn">none</span>';
			}
			//	String End //IP
			$IPstart = strpos($string, 'ip:') + strlen('ip:');
			$ip = substr($string, $IPstart);
			$this->dataList['start'] = $stringStart;
			$this->dataList['usgn'] = $usgn;
			$this->dataList['ip'] = $ip;
			// Echo dat shit.
			echo '<div class="line-number col-1-2">'.$this->lc.'</div><div class="line col-2-2">'.$this->dataList['start'].' usgn: '.$this->dataList['usgn'].', ip: '.$this->dataList['ip'].'<span class=\'finfo\'> ( '.$this->file.' :: '.$this->linecount.' :: <span class\'l-type\'>'.$this->logType.'</span> )</div><br/>';
		}

		
	}
	protected function filter_text($string, $term, $casesensitive){
		if($casesensitive == 'true'){
			$casesensitive = true;
		}else{
			$casesensitive = false;
		}
		$f = scandir($GLOBALS['paths']['logs']);
		log::debug('Extracting Search String: '.$string.' :: '.$term.' :: '.$casesensitive);
		for($i=2;$i<count($f);$i++)
		{
			$file = fopen($GLOBALS['paths']['logs'].'/'.$f[$i], 'r');
			if($file)
			{
				$this->file = $f[$i];
				log::debug('Reading '.$this->file);
				while(($line = fgets($file)) !== false)
				{
					$this->linecount = $this->linecount + substr_count($line, PHP_EOL);
					if(preg_match('/'.$string.'/', $line))
					{
						$output = fopen($GLOBALS['paths']['search'], FILEFLAG);
						//filter crap
						fputs($output, $line."\n");
						self::extractString(substr($line, TIMESTAMP_LEN), $string, $term, $casesensitive);
					}
				}
				$this->linecount=0;
				fclose($file);
			}else{
				log::error('Error Opening: '.$f[$i]);
			}
		}
	}
							

	protected function extractString($line, $string, $term, $cs){
		if(!preg_match('/^LUA ERROR: /i', $line) && !preg_match('/usgn:/i', $line) && !preg_match('/^WARNING: /', $line) && !preg_match('/^Checking: /', $line) && !array_key_exists($line, $this->filterArr)){
			if($cs){
				if($term == SEARCH_ALL){
					if(preg_match('/'.$string.'/', $line)){
						$this->lc++;
						$highlight = substr($line, strpos($line, $string), strlen($string));
						$beforeHighlight = substr($line, 0, strpos($line, $string));
						$afterHighlight = substr($line, strpos($line, $string) + strlen($string));
						echo '<div class="line-number col-1-2">'.$this->lc.'</div><div class=\'line col-2-2\'>'.$beforeHighlight.'<span class=\'highlight\'>'.$highlight.'</span>'.$afterHighlight.'<span class=\'finfo\'> ( '.$this->file.' :: '.$this->linecount.' )</div><br/>';

					}
				}elseif($term == SEARCH_ACCURATE){
					if(preg_match('/\b'.$string.'\b/', $line)){
						$this->lc++;
						$highlight = substr($line, strpos($line, $string), strlen($string));
						$beforeHighlight = substr($line, 0, strpos($line, $string));
						$afterHighlight = substr($line, strpos($line, $string) + strlen($string));
						echo '<div class="line-number col-1-2">'.$this->lc.'</div><div class=\'line col-2-2\'>'.$beforeHighlight.'<span class=\'highlight\'>'.$highlight.'</span>'.$afterHighlight.'<span class=\'finfo\'> ( '.$this->file.' :: '.$this->linecount.' )</div><br/>';
					}
				}
			}else{
				if($term == SEARCH_ALL){
					if(preg_match('/'.$string.'/i', $line)){
						$this->lc++;
						$highlight = substr($line, strpos($line, $string), strlen($string));
						$beforeHighlight = substr($line, 0, strpos($line, $string));
						$afterHighlight = substr($line, strpos($line, $string) + strlen($string));
						echo '<div class="line-number col-1-2">'.$this->lc.'</div><div class=\'line col-2-2\'>'.$beforeHighlight.'<span class=\'highlight\'>'.$highlight.'</span>'.$afterHighlight.'<span class=\'finfo\'> ( '.$this->file.' :: '.$this->linecount.' )</div><br/>';
					}
				}elseif($term == SEARCH_ACCURATE){
					if(preg_match('/\b'.$string.'\b/i', $line)){
						$this->lc++;
						$highlight = substr($line, strpos($line, $string), strlen($string));
						$beforeHighlight = substr($line, 0, strpos($line, $string));
						$afterHighlight = substr($line, strpos($line, $string) + strlen($string));
						echo '<div class="line-number col-1-2">'.$this->lc.'</div><div class=\'line col-2-2\'>'.$beforeHighlight.'<span class=\'highlight\'>'.$highlight.'</span>'.$afterHighlight.'<span class=\'finfo\'> ( '.$this->file.' :: '.$this->linecount.' )</div><br/>';
					}			
				}
			}
		}
	}



}
/*
	protected function findText($string){
			$this->string = $string;
			$lc = 0;
			$f = scandir($GLOBALS['paths']['logs']);
			$output = fopen($GLOBALS['paths']['output'], 'w');
			fputs($output, "--------------------\nSearch Term: ".$this->string."\n--------------------\n");
			fclose($output);
			for($i = 2; $i < count($f); $i++){
				$linecount = 0;
				$file = fopen(($GLOBALS['paths']['logs']."/".$f[$i]), 'r');
				if ($file) {
				    while (($line = fgets($file)) !== false) {
						$linecount = $linecount + substr_count($line, PHP_EOL);
						if (preg_match("/".$this->string."/", $line)) {
							$output = fopen($GLOBALS['paths']['output'], FILEFLAG);
							$lineOutput = substr($line, 11);
							$lc++;
							$lineput = substr($line, 11)." ( <span class='finfo'>".$f[$i]." : ".$linecount."</span> )";
							fputs($output, $lineOutput);
							$highlight = substr($line, strpos($line, $this->string), strlen($this->string));
							$line = substr($line, 11);
							$beforeHighlight = substr($line, 0, strpos($line, $this->string));
							$afterHighlight = substr($line, strpos($line, $this->string) + strlen($this->string));
							$lineput = substr($lineput, strlen($line));
							echo "<div class=\"line-number col-1-2\">".$lc."</div><div class=\"line col-2-2\">".$beforeHighlight."<span class='highlight'>".$highlight."</span>".$afterHighlight."".$lineput."</div>";
						}
				    }
				    fclose($file);

				} else {
				    // error opening the file. 
				} 
			}
			//echo preg_last_error(); # check for preg extracting errors

	}
*/
?>